@@ -11,33 +11,25 @@
 # FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
+"""Credential Plugins
 """
-$Id:$
-"""
-__docformat__ = "reStructuredText"
-
 import base64
-import transaction
-import persistent
-from urllib import urlencode
-from urllib import quote
 
+import persistent
+import transaction
 import zope.interface
+from zope.component import hooks
+from zope.container import contained
 from zope.publisher.interfaces.http import IHTTPRequest
 from zope.session.interfaces import ISession
 from zope.traversing.browser.absoluteurl import absoluteURL
 
-from zope.site import hooks
-from zope.container import contained
-
 from z3c.authenticator import interfaces
 
 
+@zope.interface.implementer(interfaces.IHTTPBasicAuthCredentialsPlugin)
 class HTTPBasicAuthCredentialsPlugin(persistent.Persistent,
-    contained.Contained):
-
-
-    zope.interface.implements(interfaces.IHTTPBasicAuthCredentialsPlugin)
+                                     contained.Contained):
 
     realm = 'Zope Application Management'
 
@@ -50,30 +42,30 @@ class HTTPBasicAuthCredentialsPlugin(persistent.Persistent,
 
           >>> from zope.publisher.browser import TestRequest
           >>> request = TestRequest(
-          ...     environ={'HTTP_AUTHORIZATION': u'Basic bWdyOm1ncnB3'})
+          ...     environ={'HTTP_AUTHORIZATION': 'Basic bWdyOm1ncnB3'})
 
         Now create the plugin and get the credentials.
 
           >>> plugin = HTTPBasicAuthCredentialsPlugin()
-          >>> plugin.extractCredentials(request)
-          {'login': u'mgr', 'password': u'mgrpw'}
+          >>> sorted(plugin.extractCredentials(request).items())
+          [('login', 'mgr'), ('password', 'mgrpw')]
 
         Make sure we return `None`, if no authentication header has been
         specified.
 
-          >>> print plugin.extractCredentials(TestRequest())
+          >>> print(plugin.extractCredentials(TestRequest()))
           None
 
         Also, this plugin can *only* handle basic authentication.
 
           >>> request = TestRequest(environ={'HTTP_AUTHORIZATION': 'foo bar'})
-          >>> print plugin.extractCredentials(TestRequest())
+          >>> print(plugin.extractCredentials(TestRequest()))
           None
 
         This plugin only works with HTTP requests.
 
           >>> from zope.publisher.base import TestRequest
-          >>> print plugin.extractCredentials(TestRequest('/'))
+          >>> print(plugin.extractCredentials(TestRequest('/')))
           None
 
         """
@@ -81,9 +73,11 @@ class HTTPBasicAuthCredentialsPlugin(persistent.Persistent,
             return None
 
         if request._auth:
-            if request._auth.lower().startswith(u'basic '):
+            if request._auth.lower().startswith('basic '):
                 credentials = request._auth.split()[-1]
-                login, password = base64.decodestring(credentials).split(':')
+                if isinstance(credentials, str):
+                    credentials = credentials.encode('utf-8')
+                login, password = base64.decodebytes(credentials).split(b':')
                 return {'login': login.decode('utf-8'),
                         'password': password.decode('utf-8')}
         return None
@@ -115,7 +109,7 @@ class HTTPBasicAuthCredentialsPlugin(persistent.Persistent,
           >>> from zope.publisher.base import TestRequest
           >>> request = TestRequest('/')
           >>> response = request.response
-          >>> print plugin.challenge(request)
+          >>> print(plugin.challenge(request))
           False
 
         """
@@ -139,7 +133,8 @@ class HTTPBasicAuthCredentialsPlugin(persistent.Persistent,
         return False
 
 
-class SessionCredentials(object):
+@zope.interface.implementer(interfaces.ISessionCredentials)
+class SessionCredentials:
     """Credentials class for use with sessions.
 
     A session credential is created with a login and a password:
@@ -156,28 +151,29 @@ class SessionCredentials(object):
       'tiger'
 
     """
-    zope.interface.implements(interfaces.ISessionCredentials)
 
     def __init__(self, login, password):
         self.login = login
         self.password = password
 
-    def getLogin(self): return self.login
+    def getLogin(self):
+        return self.login
 
-    def getPassword(self): return self.password
+    def getPassword(self):
+        return self.password
 
-    def __str__(self): return self.getLogin() + ':' + self.getPassword()
+    def __str__(self):
+        return self.getLogin() + ':' + self.getPassword()
 
 
+@zope.interface.implementer(interfaces.ISessionCredentialsPlugin)
 class SessionCredentialsPlugin(persistent.Persistent, contained.Contained):
     """A credentials plugin that uses Zope sessions to get/store credentials.
 
     To illustrate how a session plugin works, we'll first setup some session
     machinery:
 
-      >>> from zope.app.testing import placelesssetup
       >>> from z3c.authenticator.testing import sessionSetUp
-      >>> placelesssetup.setUp()
       >>> sessionSetUp()
 
     This lets us retrieve the same session info from any test request, which
@@ -194,39 +190,39 @@ class SessionCredentialsPlugin(persistent.Persistent, contained.Contained):
     If the given extractCredentials argument doesn't provide IHTTPRequest the
     result will always be None:
 
-      >>> print plugin.extractCredentials(None)
+      >>> print(plugin.extractCredentials(None))
       None
 
     Our test environment is initially configured without credentials:
 
-      >>> from zope.publisher.tests.httprequest import TestRequest
+      >>> from zope.publisher.browser import TestRequest
       >>> request = TestRequest()
-      >>> print plugin.extractCredentials(request)
+      >>> print(plugin.extractCredentials(request))
       None
 
     We must explicitly provide credentials once so the plugin can store
     them in a session:
 
-      >>> request = TestRequest(login='scott', password='tiger')
-      >>> plugin.extractCredentials(request)
-      {'login': 'scott', 'password': 'tiger'}
+      >>> request = TestRequest(form=dict(login='scott', password='tiger'))
+      >>> sorted(plugin.extractCredentials(request).items())
+      [('login', 'scott'), ('password', 'tiger')]
 
     Subsequent requests now have access to the credentials even if they're
     not explicitly in the request:
 
-      >>> plugin.extractCredentials(TestRequest())
-      {'login': 'scott', 'password': 'tiger'}
+      >>> sorted(plugin.extractCredentials(request).items())
+      [('login', 'scott'), ('password', 'tiger')]
 
     We can always provide new credentials explicitly in the request:
 
-      >>> plugin.extractCredentials(TestRequest(
-      ...     login='harry', password='hirsch'))
-      {'login': 'harry', 'password': 'hirsch'}
+      >>> sorted(plugin.extractCredentials(TestRequest(
+      ...     login='harry', password='hirsch')).items())
+      [('login', 'harry'), ('password', 'hirsch')]
 
     and these will be used on subsequent requests:
 
-      >>> plugin.extractCredentials(TestRequest())
-      {'login': 'harry', 'password': 'hirsch'}
+      >>> sorted(plugin.extractCredentials(TestRequest()).items())
+      [('login', 'harry'), ('password', 'hirsch')]
 
     We can also change the fields from which the credentials are extracted:
 
@@ -240,8 +236,8 @@ class SessionCredentialsPlugin(persistent.Persistent, contained.Contained):
 
     The plugin now extracts the credentials information from these new fields:
 
-      >>> plugin.extractCredentials(request)
-      {'login': 'luke', 'password': 'the_force'}
+      >>> sorted(plugin.extractCredentials(request).items())
+      [('login', 'luke'), ('password', 'the_force')]
 
     We can also set prefixes for the fields from which the credentials are
     extracted:
@@ -260,8 +256,8 @@ class SessionCredentialsPlugin(persistent.Persistent, contained.Contained):
 
     The plugin now extracts the credentials information from these new fields:
 
-      >>> plugin.extractCredentials(request)
-      {'login': 'harry', 'password': 'potter'}
+      >>> sorted(plugin.extractCredentials(request).items())
+      [('login', 'harry'), ('password', 'potter')]
 
     Finally, we clear the session credentials using the logout method.
     If the given logout argument doesn't provide IHTTPRequest the
@@ -277,11 +273,10 @@ class SessionCredentialsPlugin(persistent.Persistent, contained.Contained):
 
     After logout we can not logaout again:
 
-      >>> print plugin.extractCredentials(TestRequest())
+      >>> print(plugin.extractCredentials(TestRequest()))
       None
 
     """
-    zope.interface.implements(interfaces.ISessionCredentialsPlugin)
 
     challengeProtocol = None
 
@@ -321,12 +316,10 @@ class SessionCredentialsPlugin(persistent.Persistent, contained.Contained):
     def challenge(self, request):
         """Challenges by redirecting to a login form.
 
-        To illustrate how a session plugin works, we'll first setup some session
-        machinery:
+        To illustrate how a session plugin works, we'll first setup some
+        session machinery:
 
-          >>> from zope.app.testing import placelesssetup
           >>> from z3c.authenticator.testing import sessionSetUp
-          >>> placelesssetup.setUp()
           >>> sessionSetUp()
 
         and we'll create a test request:
@@ -386,8 +379,8 @@ class SessionCredentialsPlugin(persistent.Persistent, contained.Contained):
           ...     'QUERY_STRING': 'q=value'
           ...     }
           >>> request = TestRequest(environ=env)
-          >>> request._traversed_names = [u'foo', u'bar']
-          >>> request._traversal_stack = [u'page 1.html', u'folder']
+          >>> request._traversed_names = ['foo', 'bar']
+          >>> request._traversal_stack = ['page 1.html', 'folder']
           >>> request['REQUEST_URI']
           '/foo/bar/folder/page%201.html?q=value'
 
@@ -406,7 +399,7 @@ class SessionCredentialsPlugin(persistent.Persistent, contained.Contained):
           >>> session = ISession(request, None)
           >>> sessionData = session['z3c.authenticator.credential.session']
           >>> sessionData['camefrom']
-          u'/foo/bar/folder/page%201.html?q=value'
+          '/foo/bar/folder/page%201.html?q=value'
 
         If the given challenge argument doesn't provide IHTTPRequest the
         result will always be False:
@@ -430,7 +423,7 @@ class SessionCredentialsPlugin(persistent.Persistent, contained.Contained):
         camefrom = '/'.join([request.getURL(path_only=True)] + stack)
         if query:
             camefrom = camefrom + '?' + query
-        url = '%s/@@%s' % (absoluteURL(site, request), self.loginpagename)
+        url = '{}/@@{}'.format(absoluteURL(site, request), self.loginpagename)
         # only redirect to the login form
         request.response.redirect(url)
         # and store the camefrom url into a session variable, then this url
