@@ -11,6 +11,15 @@
 # FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
+import doctest
+import unittest
+
+import zope.component.testing
+import zope.password.testing
+import zope.site.testing
+from z3c.testing import BaseTestIContainer
+from z3c.testing import InterfaceBaseTest
+
 from z3c.authenticator import authentication
 from z3c.authenticator import credential
 from z3c.authenticator import group
@@ -18,11 +27,6 @@ from z3c.authenticator import interfaces
 from z3c.authenticator import principal
 from z3c.authenticator import testing
 from z3c.authenticator import user
-from z3c.testing import BaseTestIContainer
-from z3c.testing import InterfaceBaseTest
-from zope.app.testing import placelesssetup
-import doctest
-import unittest
 
 
 class AuthenticatorTest(BaseTestIContainer):
@@ -33,6 +37,11 @@ class AuthenticatorTest(BaseTestIContainer):
     def getTestClass(self):
         return authentication.Authenticator
 
+    def test_exception_in_subscriber_leaves_item_in_place(self):
+        # does the container test register a subscribe handler and not tear
+        # down?
+        pass
+
 
 class UserContainerTest(InterfaceBaseTest):
 
@@ -55,7 +64,7 @@ class UserTest(InterfaceBaseTest):
         return user.User
 
     def getTestPos(self):
-        return (u'login', u'password', u'Title')
+        return ('login', 'password', 'Title')
 
 
 class AuthenticatedPrincipalTest(InterfaceBaseTest):
@@ -70,7 +79,7 @@ class AuthenticatedPrincipalTest(InterfaceBaseTest):
         return principal.AuthenticatedPrincipal
 
     def makeTestObject(self):
-        usr =  user.User(u'login', u'password', u'Title')
+        usr = user.User('login', 'password', 'Title')
         return principal.AuthenticatedPrincipal(usr)
 
 
@@ -86,7 +95,7 @@ class FoundPrincipalTest(InterfaceBaseTest):
         return principal.FoundPrincipal
 
     def makeTestObject(self):
-        usr =  user.User(u'login', u'password', u'Title')
+        usr = user.User('login', 'password', 'Title')
         return principal.FoundPrincipal(usr)
 
 
@@ -117,7 +126,7 @@ class SessionCredentialsTest(InterfaceBaseTest):
         return credential.SessionCredentials
 
     def getTestPos(self):
-        return (u'login', u'password')
+        return ('login', 'password')
 
 
 class SessionCredentialsPluginTest(InterfaceBaseTest):
@@ -139,30 +148,46 @@ class SessionCredentialsPluginFormTest(InterfaceBaseTest):
 
 
 def test_suite():
+    loadTestsFromTestCase = unittest.defaultTestLoader.loadTestsFromTestCase
     return unittest.TestSuite((
-        doctest.DocFileSuite('README.txt',
-            setUp=testing.placefulSetUp, tearDown=testing.placefulSetUp,
-            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS),
-        doctest.DocFileSuite('group.txt',
-            setUp=testing.placefulSetUp, tearDown=testing.placefulSetUp,
-            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS),
-        doctest.DocTestSuite('z3c.authenticator.credential',
-            setUp=placelesssetup.setUp, tearDown=placelesssetup.tearDown),
-        doctest.DocTestSuite('z3c.authenticator.group',
-            setUp=placelesssetup.setUp, tearDown=placelesssetup.tearDown),
-        doctest.DocFileSuite('vocabulary.txt',
-            setUp=placelesssetup.setUp, tearDown=placelesssetup.tearDown),
-        unittest.makeSuite(AuthenticatorTest),
-        unittest.makeSuite(UserContainerTest),
-        unittest.makeSuite(UserTest),
-        unittest.makeSuite(AuthenticatedPrincipalTest),
-        unittest.makeSuite(FoundPrincipalTest),
-        unittest.makeSuite(GroupContainerTest),
-        unittest.makeSuite(GroupTest),
-        unittest.makeSuite(SessionCredentialsTest),
-        unittest.makeSuite(SessionCredentialsPluginTest),
-        unittest.makeSuite(SessionCredentialsPluginFormTest),
-        ))
-
-if __name__ == '__main__':
-    unittest.main(defaultTest='test_suite')
+        doctest.DocFileSuite(
+            'README.txt',
+            setUp=testing.placefulSetUp,
+            tearDown=testing.placefulTearDown,
+            optionflags=doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS),
+        doctest.DocFileSuite(
+            'group.txt',
+            setUp=testing.placefulSetUp,
+            tearDown=testing.placefulTearDown,
+            optionflags=doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS),
+        doctest.DocTestSuite(
+            'z3c.authenticator.credential',
+            setUp=testing.placefulSetUp,
+            tearDown=testing.placefulTearDown),
+        doctest.DocTestSuite(
+            'z3c.authenticator.user',
+            setUp=testing.placefulSetUp,
+            tearDown=testing.placefulTearDown),
+        doctest.DocTestSuite(
+            'z3c.authenticator.event',
+            setUp=testing.placefulSetUp,
+            tearDown=testing.placefulTearDown),
+        doctest.DocTestSuite(
+            'z3c.authenticator.group',
+            setUp=testing.placefulSetUp,
+            tearDown=testing.placefulTearDown),
+        doctest.DocFileSuite(
+            'vocabulary.txt',
+            setUp=zope.component.testing.setUp,
+            tearDown=zope.component.testing.tearDown),
+        loadTestsFromTestCase(AuthenticatorTest),
+        loadTestsFromTestCase(UserContainerTest),
+        loadTestsFromTestCase(UserTest),
+        loadTestsFromTestCase(AuthenticatedPrincipalTest),
+        loadTestsFromTestCase(FoundPrincipalTest),
+        loadTestsFromTestCase(GroupContainerTest),
+        loadTestsFromTestCase(GroupTest),
+        loadTestsFromTestCase(SessionCredentialsTest),
+        loadTestsFromTestCase(SessionCredentialsPluginTest),
+        loadTestsFromTestCase(SessionCredentialsPluginFormTest),
+    ))
