@@ -11,53 +11,45 @@
 # FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
+"""Users
 """
-$Id:$
-"""
-__docformat__ = "reStructuredText"
-
-try:
-    from hashlib import md5
-except ImportError: # BBB for Python 2.4
-    from md5 import md5
 import random
-import time
 import socket
+import time
+from hashlib import md5
+
 import persistent
-import zope.interface
 import zope.component
-from zope.container import contained
+import zope.interface
 from zope.container import btree
+from zope.container import contained
 from zope.container.interfaces import DuplicateIDError
 from zope.password.interfaces import IPasswordManager
 
 from z3c.authenticator import interfaces
 
-# get the IP addresss only once
+
+# get the IP address only once
 try:
     ip = socket.getaddrinfo(socket.gethostname(), 0)[-1][-1][0]
-except:
+except BaseException:
     ip = '127.0.0.1'
 
+
 def generateUserIDToken(id):
     """Generates a unique user id token."""
-    t = long(time.time() * 1000)
-    r = long(random.random()*100000000000000000L)
-    try:
-        id = str(id)
-    except UnicodeEncodeError:
-        id = id.encode('utf-8')
-    data = str(ip)+' '+str(t)+' '+str(r)+' '+id
-    return unicode(md5(data).hexdigest())
+    t = int(time.time() * 1000)
+    r = int(random.random() * 100000000000000000)
+    data = "{} {} {} {}".format(ip, t, r, id)
+    return md5(data.encode('utf-8')).hexdigest()
 
 
+@zope.interface.implementer(interfaces.IUser)
 class User(persistent.Persistent, contained.Contained):
     """User stored in IUserContainer."""
 
-    zope.interface.implements(interfaces.IUser)
-
-    def __init__(self, login, password, title, description=u'',
-            passwordManagerName="Plain Text"):
+    def __init__(self, login, password, title, description='',
+                 passwordManagerName="Plain Text"):
         self._login = login
         self._passwordManagerName = passwordManagerName
         self.password = password
@@ -71,7 +63,7 @@ class User(persistent.Persistent, contained.Contained):
 
     def _getPasswordManager(self):
         return zope.component.getUtility(IPasswordManager,
-            self.passwordManagerName)
+                                         self.passwordManagerName)
 
     def getPassword(self):
         return self._password
@@ -104,16 +96,15 @@ class User(persistent.Persistent, contained.Contained):
     login = property(getLogin, setLogin)
 
 
+@zope.interface.implementer(interfaces.IUserContainer)
 class UserContainer(btree.BTreeContainer):
     """A Persistent User Container and authenticator plugin.
 
     See principalfolder.txt for details.
     """
 
-    zope.interface.implements(interfaces.IUserContainer)
-
     def __init__(self):
-        super(UserContainer, self).__init__()
+        super().__init__()
         self.__id_by_login = self._newContainerData()
 
     def notifyLoginChanged(self, oldLogin, principal):
@@ -133,56 +124,62 @@ class UserContainer(btree.BTreeContainer):
 
         Create a UserContainer
 
-        >>> mc = UserContainer('users.')
+        >>> mc = UserContainer()
 
         Try to add something not providing IUser
         >>> try:
         ...     mc.__setitem__(u'max', object())
-        ... except Exception, e:
-        ...     print e
+        ... except Exception as e:
+        ...     print(e)
         Item does not support IUser!
 
         Create a user with no __name__, this should be added via the add
         method
 
-        >>> user = User()
+        >>> user = User(u'max', u'passwd', u'sir')
         >>> try:
         ...     mc.__setitem__(u'max', user)
-        ... except Exception, e:
-        ...     print e
+        ... except Exception as e:
+        ...     print(e)
         There is no user id token given!
 
         Probably we do have a __name__ during copy/paste, so we have to check
         if we get a __parent__ as well
 
-        >>> user = User()
+        >>> user = User(u'usertoken', u'passwd', u'sir')
         >>> user.__name__ = u'usertoken'
+        >>> user.__parent__ = u'parent'
         >>> try:
-        ...     mc.__setitem__(u'max', user)
-        ... except Exception, e:
-        ...     print e
+        ...     mc.__setitem__(u'usertoken', user)
+        ... except Exception as e:
+        ...     print(e)
         Paste a object is not supported!
 
         Try to use a user with no login:
 
+        >>> user = User(u'', u'passwd', u'sir')
+        >>> user.__name__ = u''
         >>> try:
-        ...     mc.__setitem__(u'max', user)
-        ... except Exception, e:
-        ...     print e
+        ...     mc.__setitem__(u'', user)
+        ... except Exception as e:
+        ...     print(e)
         User does not provide a login value!
 
         Add a login attr since __setitem__ is in need of one
 
-        >>> user.login = u'max'
-        >>> mc.__setitem__(u'users.max', principal)
+        >>> user = User(u'max', u'passwd', u'sir')
+        >>> user.__name__ = u'max'
+        >>> mc.__setitem__(u'max', user)
 
         Now try to use the same login:
 
+        >>> user2 = User(u'max', u'passwd', u'sir')
+        >>> user2.__name__ = u'max'
         >>> try:
-        ...     mc.__setitem__(u'max', user)
-        ... except Exception, e:
-        ...     print e
-        Login already taken!
+        ...     mc.__setitem__(u'max', user2)
+        ... except Exception as e:
+        ...     print(e)
+        'Login already taken!'
         """
 
         # check if we store correct implementations
@@ -190,7 +187,7 @@ class UserContainer(btree.BTreeContainer):
             raise TypeError('Item does not support IUser!')
 
         # check if there is a user id given
-        if user.__name__ is not id:
+        if user.__name__ != id:
             raise ValueError('There is no user id token given!')
 
         # check if there is a user id given in we store correct implementations
@@ -205,7 +202,7 @@ class UserContainer(btree.BTreeContainer):
         if user.login in self.__id_by_login:
             raise DuplicateIDError('Login already taken!')
 
-        super(UserContainer, self).__setitem__(id, user)
+        super().__setitem__(id, user)
         self.__id_by_login[user.login] = id
 
     def add(self, user):
@@ -218,7 +215,7 @@ class UserContainer(btree.BTreeContainer):
     def __delitem__(self, id):
         """Remove principal information."""
         user = self[id]
-        super(UserContainer, self).__delitem__(id)
+        super().__delitem__(id)
         del self.__id_by_login[user.login]
 
     def authenticateCredentials(self, credentials):
@@ -237,7 +234,7 @@ class UserContainer(btree.BTreeContainer):
         return user
 
     def getUserByLogin(self, login):
-        #don't bother catching KeyError, it's the task of the caller
+        # don't bother catching KeyError, it's the task of the caller
         return self[self.__id_by_login[login]]
 
     def queryPrincipal(self, id, default=None):
@@ -256,7 +253,7 @@ class UserContainer(btree.BTreeContainer):
         for i, value in enumerate(self.values()):
             if (search in value.title.lower() or
                 search in value.description.lower() or
-                search in value.login.lower()):
+                    search in value.login.lower()):
                 if not ((start is not None and i < start)
                         or (batch_size is not None and n > batch_size)):
                     n += 1
